from .dfkv import dfkv as kv
import dfkv.common as common
import dfkv.ext as ext
import dfkv.dt as dt

__title__ = 'dfkv'
__author__ = 'Fred Deng'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2023-2023 Fred Deng'



from datetime import datetime, date
import time

def now(type = 1) -> str:
    """1: 2023-04-06 15:38:34.267
    2: 2023-04-06 15:38:34.267
    3: 20230406153834267
    4: 20230406153834
    5: 2023-04-06
    6: 20230406
    """
    if type == 1:
        return datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]
    elif type == 2:
        return datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    elif type == 3:
        return datetime.now().strftime("%Y%m%d%H%M%S%f")[:-3]
    elif type == 4:
        return datetime.now().strftime("%Y%m%d%H%M%S")
    elif type == 5:
        return date.today().strftime("%Y-%m-%d")
    elif type == 6:
        return date.today().strftime("%Y%m%d")
    
    else:
        return datetime.now()

def timestamp(type = 1) -> int:
    """
    1: 1686615980262 -> (ms)
    2: 1686616065 -> (s)
    """
    if type == 1:
        return int(time.time() * 1000)
    elif type == 2:
        return int(time.time())
    else:
        return time.time()
if __name__ == "__main__":
    print(timestamp(), timestamp(2), timestamp())


class dfkv(dict):
    # def has(self, message):
    #     print(f'this is dfkv.has(). {message= }')
    """
    Get attributes

    >>> d = dfkv({'foo':3})
    >>> d['foo']
    3
    >>> d.foo
    3
    >>> d.bar
    Traceback (most recent call last):
    ...
    AttributeError: 'dfkv' object has no attribute 'bar'

    Works recursively

    >>> d = dfkv({'foo':3, 'bar':{'x':1, 'y':2}})
    >>> isinstance(d.bar, dict)
    True
    >>> d.bar.x
    1

    Bullet-proof

    >>> dfkv({})
    {}
    >>> dfkv(d={})
    {}
    >>> dfkv(None)
    {}
    >>> d = {'a': 1}
    >>> dfkv(**d)
    {'a': 1}
    >>> dfkv((('a', 1), ('b', 2)))
    {'a': 1, 'b': 2}
    
    Set attributes

    >>> d = dfkv()
    >>> d.foo = 3
    >>> d.foo
    3
    >>> d.bar = {'prop': 'value'}
    >>> d.bar.prop
    'value'
    >>> d
    {'foo': 3, 'bar': {'prop': 'value'}}
    >>> d.bar.prop = 'newer'
    >>> d.bar.prop
    'newer'


    Values extraction

    >>> d = dfkv({'foo':0, 'bar':[{'x':1, 'y':2}, {'x':3, 'y':4}]})
    >>> isinstance(d.bar, list)
    True
    >>> from operator import attrgetter
    >>> list(map(attrgetter('x'), d.bar))
    [1, 3]
    >>> list(map(attrgetter('y'), d.bar))
    [2, 4]
    >>> d = dfkv()
    >>> list(d.keys())
    []
    >>> d = dfkv(foo=3, bar=dict(x=1, y=2))
    >>> d.foo
    3
    >>> d.bar.x
    1

    Still like a dict though

    >>> o = dfkv({'clean':True})
    >>> list(o.items())
    [('clean', True)]

    And like a class

    >>> class Flower(dfkv):
    ...     power = 1
    ...
    >>> f = Flower()
    >>> f.power
    1
    >>> f = Flower({'height': 12})
    >>> f.height
    12
    >>> f['power']
    1
    >>> sorted(f.keys())
    ['height', 'power']

    update and pop items
    >>> d = dfkv(a=1, b='2')
    >>> e = dfkv(c=3.0, a=9.0)
    >>> d.update(e)
    >>> d.c
    3.0
    >>> d['c']
    3.0
    >>> d.get('c')
    3.0
    >>> d.update(a=4, b=4)
    >>> d.b
    4
    >>> d.pop('a')
    4
    >>> d.a
    Traceback (most recent call last):
    ...
    AttributeError: 'dfkv' object has no attribute 'a'
    """
    def __init__(self, d=None, **kwargs):
        if d is None:
            d = {}
        else:
            d = dict(d)        
        if kwargs:
            d.update(**kwargs)
        for k, v in d.items():
            setattr(self, k, v)
        # Class attributes
        for k in self.__class__.__dict__.keys():
            if not (k.startswith('__') and k.endswith('__')) and not k in ('update', 'pop'):
                setattr(self, k, getattr(self, k))

    def __setattr__(self, name, value):
        if isinstance(value, (list, tuple)):
            value = [self.__class__(x)
                     if isinstance(x, dict) else x for x in value]
        elif isinstance(value, dict) and not isinstance(value, self.__class__):
            value = self.__class__(value)
        super(dfkv, self).__setattr__(name, value)
        super(dfkv, self).__setitem__(name, value)

    __setitem__ = __setattr__

    def update(self, e=None, **f):
        d = e or dict()
        d.update(f)
        for k in d:
            setattr(self, k, d[k])

    def pop(self, k, d=None):
        delattr(self, k)
        return super(dfkv, self).pop(k, d)


#!/usr/bin/env python
from io import open
from setuptools import setup, find_packages
# from distutils.core import setup
# setup(name="dfkv", version="0.0.6", description="test to install module", author="someone", py_modules=['dfkv.common'])
setup(
    name='dfkv',
    version='0.0.9',
    description='a expand for dict',
    long_description='expand dict, can use a.b, can set value more than 1-level a.b.c.d="xyz"',
    author='fred deng',
    author_email='dfgeoff@qq.com',
    license='Apache License 2.0',
    url='https://gitee.com/hifong45/dkv.git',
    download_url='https://gitee.com/hifong45/dkv/master.zip',
    packages=find_packages(exclude=["examples", "project", "tests", "*.tests", "*.tests.*"]),  # 必填
    # package_dir={'': 'src'},  # 必填
    install_requires=[]
)
# 本地测试时，使用： rd /s /q dist && python setup.py sdist && pip uninstall -y dfkv && pip install dist\dfkv-0.0.9.tar.gz
# 要上传时，用：rd /s /q dist && python setup.py sdist && twine upload dist/*
# 指定用标准中心库： pip install -i https://pypi.Python.org/simple/ dfkv
# 2023-01-28 16:16:52.993077